<script language="php">
// ######################################################################
//
// Path: /usr/local/includes/php/roma/romadom.php
//
// ######################################################################

/**
 * This class is responsible for Romas customization file. 
 *
 * @author: Arno Mittelbach <arno@mittelbach-online.de>
 * @version: 0.9 (CVS $Id: romadom.php 2460 2007-05-19 15:18:56Z rahtz $)
 * @access:  public
 * @package: roma
 */
class romaDom extends domDocument
  {
    function __construct( $szXML = '')
      {
	parent::__construct();

	$this->preserveWhiteSpace = false;

	if ( $szXML == '' )
	  {
 	    $this->constructDocument();
          }
	else
	  {
	    $this->loadXML( $szXML );
          }
      }

    private function constructDocument()
      {
        $oTemp = new domDocument();
	$oTEI = $oTemp->createElementNS( 'http://www.tei-c.org/ns/1.0', 'TEI' );
	$oTemp->appendChild( $oTEI );
	$oTEI->setAttributeNS('http://www.w3.org/XML/1998/namespace', 'xml:lang', 'en' );

	$oTeiHeader = $oTEI->appendChild( new domElement( 'teiHeader' ) );

	$oFileDesc = $oTeiHeader->appendChild( new domElement( 'fileDesc' ) );

	$oTitleStmt = $oFileDesc->appendChild( new domElement( 'titleStmt' ) );
	$oTitle = $oTitleStmt->appendChild( new domElement( 'title' ) );
	$oTitle->appendChild( new domText( 'My TEI Extension' ) );
	$oAuthor = $oTitleStmt->appendChild( new domElement( 'author' ) );
	$oAuthor->appendChild( new domText( 'generated by Roma ' . roma_version ) );

	$oPublicationStmt = $oFileDesc->appendChild( new domElement( 'publicationStmt' ) );
	$oP = $oPublicationStmt->appendChild( new domElement( 'p' ) );
	$oP->appendChild( new domText( 'for use by whoever wants it' ) );

	$oSourceDesc = $oFileDesc->appendChild( new domElement( 'sourceDesc' ) );
	$oP = $oSourceDesc->appendChild( new domElement( 'p' ) );
//Tuesday 13th 2006f June 2006 10:57:23 PM
	$oP->appendChild( new domText( 'created on ' . date( "l dS F Y h:i:s A" ) . ' by the form at http://www.tei-c.org.uk/Roma/' ) );

	$oText = $oTEI->appendChild( new domElement( 'text' ) );
	$oFront = $oText->appendChild( new domElement( 'front' ) );
	$oBody = $oText->appendChild( new domElement( 'body' ) );
	$oBodyP = $oBody->appendChild( new domElement( 'p' ) );
	$oBodyP->appendChild( new domText( 'My TEI Customization
	starts with modules tei, core, header, and textstructure' ) );

	$oDivgen = $oFront->appendChild( new domElement( 'divGen' ) );
	$oDivgen->setAttribute( 'type', 'toc' );

	$oSchema = $oBody->appendChild( new domElement( 'schemaSpec' ) );
	$oSchema->setAttribute( 'ident', 'myTei' );
	$oSchema->setAttribute('docLang', 'en' );
	$oSchema->setAttributeNS('http://www.w3.org/XML/1998/namespace', 'xml:lang', 'en' );

	$this->loadXML( $oTemp->SaveXML() );
      }

    // #####################################################################
    // --- Little Helper functions
    // #####################################################################

    /**
     * Creates an XPath Object with the current customization
     * and registers the namespace tei
     */
    public function getXPath( &$oXPath )
      {
        $oXPath = new domxpath( $this );
	$oXPath->registerNamespace( 'tei', 'http://www.tei-c.org/ns/1.0' );
      }


    // #####################################################################
    // --- Get Information out of the Customization
    // #####################################################################

    /**
     * returns a numbered array with all those modules the user selected
     */
    public function getHeader( &$aszHeader )
      {
	$this->getXPath( $oXPath );
        $aszHeader = $oXPath->query( "//tei:teiHeader" );
      }

    public function getSelectedModules( &$aszModules )
      {
	$this->getXPath( $oXPath );
        $aoModules = $oXPath->query( "//tei:schemaSpec/tei:moduleRef/@key" );

	$aszModules = array();
	foreach( $aoModules as $oModule )
	  {
	    $aszModules[] = $oModule->nodeValue;
          }
      }

    /**
     * returns a dom
     */
    public function getSelectedModulesDom( &$oDom )
      {
	$this->getSelectedModules( $aszModules );
	$oDom = new domDocument();
	$oDom->appendChild( new domElement( 'selectedModules' ) );
	
	foreach( $aszModules as $szModule )
	  {
	    $oDom->documentElement->appendChild( new domElement( 'module', $szModule ) );
	  } 
      }


    /**
     * returns a numberd array with all those modules the user changed
     */
    public function getChangedModules( &$aszModules )
      {
	$this->getXPath( $oXPath );
        $aoModules = $oXPath->query("//tei:schemaSpec/tei:elementSpec[@mode='change']/@module");
	$aszModules = array();
	foreach( $aoModules as $oModule )
	  {
	    $aszModules[] = $oModule->nodeValue;
          }
      }

    /**
     * returns a dom
     */
    public function getChangedModulesDom( &$oDom )
      {
	$this->getChangedModules( $aszModules );
	$oDom = new domDocument();
	$oDom->appendChild( new domElement( 'changedModules' ) );

	foreach( $aszModules as $szModule )
	  {
	    $oDom->documentElement->appendChild( new domElement( 'module', $szModule ) );
	  }
      }


    /**
     * returns a numbered array with all excluded Elements inside a module
     * the module has to be specified with the first parameter.
     */
    public function getExcludedElementsInModule( $szModule, &$aszElements )
      {
	$this->getXPath( $oXPath );
        $aoElements = $oXPath->query(
    "//tei:schemaSpec/tei:elementSpec[@mode='delete' and @module='$szModule']/@ident" );
	
	$aszElements = array();
	foreach( $aoElements as $oElement )
	  {
	    $aszElements[] = $oElement->nodeValue;
	  }
      }

    /**
     */
    public function getExcludedElementsInModuleDom( $szModule, &$oDom )
      {
	$oDom = new domDocument();
	$oDom->appendChild( new domElement( 'excludedElements' ) );

	$this->getExcludedElementsInModule( $szModule, $aszElements );
	foreach( $aszElements as $szElement )
	  {
	    $oDom->documentElement->appendChild( new domElement( 'element', $szElement ) );
	  }
      }



    /**
     * returns a numbered array with all included Elements inside a module
     * the module has to be specified with the first parameter.
     */
    public function getIncludedElementsInModule( $szModule, &$aszElements )
      {
	$oListDom = new domDocument();
	$oListDom->loadXML( join( '', file( roma_xquery_server . 'elemsbymod.xq?module=' . $szModule ) ) );
	$aoElements = $oListDom->getElementsByTagname( 'elementName' );
	$this->getExcludedElementsInModule( $szModule, $aszExcluded );

	$aszElements = array();
	foreach( $aoElements as $oElement )
	  {
	    if( ! in_array( $oElement->nodeValue, $aszExcluded ) )
	      {
		$aszElements[] = $oElement->nodeValue;
	      }
	  }
      }


    /**
     * returns a domDocument with all added Elements.
     * returns true if the user did not yet create any new Elements.
     * The structure of the domDocument:
     *
     *   <addedElements>
     *    <Element>
     *     <elementName>Name</elementName>
     *     <include>change|delete</include>
     *     <desc>Description</desc>
     *    </Element>
     *   </addedElements>
     *
     * There might be some more tags inside the Element-tag like
     * <classes> but since they are not of interest here, they were
     * omitted. There is also just one level of Elements beyond the
     * Element-tag. So <classes> might be there, but there would be no
     * <memberOf> inside the <classes> */

    public function getAddedElements( &$oElementDom )
      {
	$errResult = false;

        $oElementDom = new domDocument;
        $oRoot = $oElementDom->appendChild( new domElement( 'addedElements' ) );

	$this->getXPath( $oXPath );
        $aoElements = $oXPath->query( "//tei:schemaSpec/tei:elementSpec[@mode='add']" );

	if ( ! is_object( $aoElements->item(0) ) )
	  {
	    $errResult = true;
	  }

        foreach( $aoElements as $oElement )
          {
	    $oElem = $oRoot->appendChild( new domElement( 'Element' ) );
	    $oName = $oElem->appendChild( new domElement( 'elementName' ) );	    
            $oName->appendChild( new domText( $oElement->getAttribute( 'ident' ) ) );
	    
	    $oInclude = $oElem->appendChild( new domElement( 'include' ) );
	    $oInclude->appendChild( new domText( $oElement->getAttribute( 'mode' ) ) );
	      

	    foreach( $oElement->childNodes as $oChild )
	      {
		if ( $oChild->nodeType == XML_ELEMENT_NODE )
		  {
		    $oElemDesc = $oElem->appendChild( new domElement( $oChild->nodeName ) );
		    $oElemDesc->appendChild( new domText( $oChild->nodeValue ) );
		  }
	      }
          }

	return $errResult;
      }

    public function getAddedElementsDescription( $szIdent, &$szDescription )
      {
	$this->getXPath( $oXPath );
        $oDesc = $oXPath->query( "//tei:schemaSpec/tei:elementSpec[@ident='$szIdent']/tei:desc" )->item(0);

        if ( is_object( $oDesc ) )
          {
            $szDescription = $oDesc->nodeValue;
          }
      }

    public function getAddedElementsContents( $szIdent, &$szContents )
      {
	$this->getXPath( $oXPath );
        $oContent = $oXPath->query( "//tei:schemaSpec/tei:elementSpec[@ident='$szIdent']/tei:content" )->item(0);

        if ( is_object( $oContent ) )
          {
	    $oRNG = $oContent->childNodes->item(0);
            switch( $oRNG->nodeName ) 
              {
                case 'rng:empty':
                  $szContents = 'empty';
                  break;
                case 'rng:text':
                  $szContents = 'text';
                  break;
                default:
		  if ( is_object( $oRNG ) )
		    {
		      $szContents = $oRNG->nodeValue;
/* ->getAttribute( 'name' ); */
		    }
                  break;
              }
          }
      }
    public function getAddedElementsFullContents( $szIdent, &$szContents )
      {
	$this->getXPath( $oXPath );
        $oContent = $oXPath->query( "//tei:schemaSpec/tei:elementSpec[@ident='$szIdent']/tei:content" )->item(0);
	$szContents = $this->SaveXML( $oContent );
      }


    public function getAddedElementsClasses( $szIdent, &$aszClasses )
      {
        $aszClasses = array();

	$this->getXPath( $oXPath );
        $aoClasses = $oXPath->query( "//tei:schemaSpec/tei:elementSpec[@ident='$szIdent']/tei:classes/tei:memberOf/@key" );
   
        foreach( $aoClasses as $oClass )
          {
            $aszClasses[] = $oClass->nodeValue;
          }        
      }

    public function getElementsWithChangedNameInModule( $szModule, &$aszElements )
      {
	$aszElements = array();

	$this->getXPath( $oXPath );
	$aoElements = $oXPath->query(
	"//tei:schemaSpec/tei:elementSpec[@mode='change' and @module='{$szModule}']/tei:altIdent" );

	foreach( $aoElements as $oElement )
	  {
	    $aszElements[ $oElement->parentNode->getAttribute( 'ident' ) ] =  $oElement->firstChild->nodeValue;
	  }
      }

    public function getElementsWithChangedNameInModuleDom( $szModule, &$oDom )
      {
	$oDom = new domDocument;
	$oDom->appendChild( new domElement( 'changedNames' ) );

	$this->getElementsWithChangedNameInModule( $szModule, $aszElements );
	foreach ( $aszElements as $szOld => $szNew )
	  {
	    $oElement = $oDom->documentElement->appendChild( new domElement( 'element' ) );
	    $oElement->setAttribute( 'ident', $szOld );
	    $oElement->appendChild( new domElement( 'altIdent', $szNew ) );
	  }
      }


    public function getElementsChangedNameInModule( $szModule, $szElement, &$szElementNew )
      {
	$this->getXPath( $oXPath );
	$oElement = $oXPath->query(
      "//tei:schemaSpec/tei:elementSpec[@module='$szModule'
      and @mode='change' and @ident='{$szElement}']/tei:altIdent" )->item(0);

	$szElementNew = ( is_object ( $oElement ) ) ? $oElement->nodeValue : $szElement;
      }

    public function getClassesByElementNameInModule( $szElement, $szModule, &$aszClasses )
      {
        $aszClasses = array();

	$this->getXPath( $oXPath );
	$oClasses = $oXPath->query(
	"//tei:schemaSpec/tei:elementSpec[@module='$szModule'
	and @mode='change' and @ident='$szElement']/tei:classes" )->item(0);

	if ( is_object( $oClasses ) )
	  {
	    foreach ( $oClasses->childNodes as $oMember )
	      {
		$aszClasses[] = $oMember->getAttribute( 'key' );
	      }
	  }
	else
	  {
	    $oElementDom = new domDocument();
	    $oElementDom->loadXML( join( '', file( roma_xquery_server
	  . 'element.xq?lang=' . $_SESSION['docLang'] . '&name=' . $szElement ) ) );
	    
	    $oXPath = new domxpath( $oElementDom );
	    $aoClasses = $oXPath->query( "/Element/elementClasses/class" );
	    
	    foreach( $aoClasses as $oClass )
	      {
		$aszClasses[] = $oClass->nodeValue;
	      }
	  }
      } 

    public function getDescriptionByElementNameInModule( $szElement,
      $szModule,  &$szDesc )
      {
	$this->getXPath( $oXPath );
	$oDesc = $oXPath->query(
      "//tei:schemaSpec/tei:elementSpec[@module='$szModule'
      and @mode='change' and @ident='$szElement']/tei:desc" )->item(0);

        if ( is_object( $oDesc ) ) 
          {
  	    $szDesc = $oDesc->nodeValue;
          }
	else
	  {
	    $this->getDescriptionByElementName($szElement, $szDesc );
	  }
      }

    public function getDescriptionByElementName($szElement, &$szDesc )
      {
	$oElementDom = new domDocument();
	$oElementDom->loadXML( join( '', file( roma_xquery_server . 'element.xq?lang=' . $_SESSION['docLang'] .'&name=' . $szElement ) ) );
	
	$oXPath = new domxpath( $oElementDom );
	$oDesc = $oXPath->query( "/Element/elementDesc" )->item(0);
	
	if ( is_object( $oDesc ) )
	  {
	    $szDesc = $oDesc->nodeValue;
	  }
      } 

    public function getContentsByElementNameInModuleDom( $szElement, $szModule, &$oContents )
      {
	$this->getXPath( $oXPath );

	$oContent = $oXPath->query(
      "//tei:schemaSpec/tei:elementSpec[@module='$szModule'
      and @mode='change' and @ident='$szElement']/tei:content" )->item(0);

        if ( is_object( $oContent ) ) 
          {
	    $oContents = new domDocument();
	    $oContent = $oContents->importNode( $oContent, true );
	    $oContents->appendChild( $oContent );
          }
	else
	  {
	    $oElementDom = new domDocument();
	    $oElementDom->loadXML( join( '', file( roma_xquery_server
          . 'element.xq?lang=' . $_SESSION['docLang'] . '&name=' . $szElement ) ) );
	    
	    $oXPath = new domxpath( $oElementDom );
	    $oXPath->registerNamespace( 'rng', 'http://relaxng.org/ns/structure/1.0' );
	    $oContent = $oXPath->query( "/Element/elementContent/child::*" )->item(0);
	    
	    if ( is_object( $oContent ) )
	      {
		$oContents = new domDocument();

                $theCon = $oContents->createElementNS( 'http://www.tei-c.org/ns/1.0', 'content' );
		$oCon = $oContents->appendChild( $theCon );
		$oContent = $oContents->importNode( $oContent, true );
		$oCon->appendChild( $oContent );
	      }
	  } 
      }
 
    public function getContentsByElementNameInModule( $szElement, $szModule, &$szContents )
      {
	$this->getXPath( $oXPath );
	$oContent = $oXPath->query(
      "//tei:schemaSpec/tei:elementSpec[@module='$szModule'
      and @mode='change' and @ident='$szElement']/tei:content" )->item(0);

        if ( is_object( $oContent ) ) 
          {
	    $oConDom = new domDocument();
	    $oContent = $oConDom->importNode( $oContent, true );
	    $oConDom->appendChild( $oContent );
	    $szContents = $oConDom->SaveHTML();
          }
	else
	  {
	    $oElementDom = new domDocument();
	    $oElementDom->loadXML( join( '', file( roma_xquery_server . 'element.xq?lang=' . $_SESSION['docLang'] . '&name=' . $szElement ) ) );
	    
	    $oXPath = new domxpath( $oElementDom );
	    $oXPath->registerNamespace( 'rng', 'http://relaxng.org/ns/structure/1.0' );
	    $oContent = $oXPath->query( "/Element/elementContent/child::*" )->item(0);
	    
	    if ( is_object( $oContent ) )
	      {
		$oDom = new domDocument();

                $theCon = $oDom->createElementNS( 'http://www.tei-c.org/ns/1.0', 'content' );
		$oCon = $oDom->appendChild( $theCon );
		$oContent = $oDom->importNode( $oContent, true );
		$oCon->appendChild( $oContent );
		
		$szContents = $oDom->SaveHTML();
	      }
	  } 
      }

    public function getAttributesByElement( $szElement, &$aszAttributes )
      {
	$aszAttributes = array();

	$oAttributesDom = new domDocument();
	$oAttributesDom->loadXML( join( '', file( roma_xquery_server
	. 'attsbyelem.xq?lang=' . $_SESSION['docLang'] . '&name=' . $szElement ) ) );
	
	$oElement = $oAttributesDom->documentElement;
	foreach( $oElement->childNodes as $oChild )
	  {
	    $aszAttributes[] = $oChild->nodeValue;
	  }
      }

    public function getAttributeDomByElementInModule( $szElement, $szModule, $szClass, &$oAttDom )
      {
	$errResult = false;

	$oAttDom = new domDocument;
	$this->getXPath( $oXPath );

	if ( $szModule != '' && $szClass == '')
	  {
// I get here if I edit an attribute on an existing element
	    @$oAttDom->loadXML( join( '', file( roma_xquery_server
	  . 'attsbyelem.xq?lang=' . $_SESSION['docLang'] . '&name=' . $szElement ) ) );
	    $oElement = $oAttDom->documentElement;
		
	    if ( is_object( $oElement ) )
	      {
		if ( ! is_object( $oElement->getElementsByTagname( 'att' )->item(0) ) )
		  {
		    $errResult = true;
		  }
	      }
	    else
	      {
		$errResult = true;
	      }

	    $oAttList = $oXPath->query(
	  "//tei:schemaSpec/tei:elementSpec[@mode='change' and
	  @ident='{$szElement}' and @module='{$szModule}']/tei:attList" )->item(0);
	  }
	elseif( $szClass == '' )
	  {
// I get here if I edit an attribute on an attribute on a new element
	    $oAttDom->appendChild( new domElement( 'Element' ) );
	    $oElement = $oAttDom->documentElement;

	    $oAttList = $oXPath->query( "//tei:schemaSpec/tei:elementSpec[@ident='{$szElement}']/tei:attList" )->item(0);
	    $errResult = true;
	  }
	else
	  {
// I get here if I look at class attributes 
	    $oAttDom->appendChild( new domElement( 'Element' ) );
	    $oElement = $oAttDom->documentElement;

	    $oAttClassDom = new domDocument;
	    $oAttClassDom->loadXML( join( '', file( roma_xquery_server . 'attclassbyname.xq?lang=' . $_SESSION['docLang'] . '&class=' . $szClass ) ) );
	    $oRoot = $oAttClassDom->documentElement;
	    $oAttributes = $oAttClassDom->getElementsByTagName( 'attributes' )->item(0);

	    foreach( $oAttributes->childNodes as $oChild )
	      {
		$oNode = $oAttDom->importNode( $oChild, true );
		$oElement->appendChild( $oNode );
	      }
	    $oAttList = $oXPath->query("//tei:schemaSpec/tei:classSpec[@module='$szModule'
	      and @mode='change' and @ident='$szClass']/tei:attList" )->item(0);
	  }

// ok we are sitting on an element. Now check its attributes.
	if ( is_object( $oAttList ) )
	  {
	    if ( $errResult && count( $oAttList->childNodes ) > 0 )
	      {
		$errResult = false;
	      }

	    $oAttXPath = new domxpath( $oAttDom );

	    foreach ( $oAttList->childNodes as $oChild )
	      {
   	        if ($oChild->nodeType != XML_ELEMENT_NODE) { continue; }
		$oAtt = $oAttXPath->query( "/Element/att[child::name[node()='" . $oChild->getAttribute( 'ident' ) . "']]" )->item(0);
		if ( ( $szModule != '' && ! is_object( $oAtt ) ) || $szModule == '' ) //if attribute was added
		  { 
		    $oAtt = $oElement->appendChild( new domElement ( 'att' ) );
		    $oAtt->setAttribute( 'added', 'true' );

		    $oAttName = $oAtt->appendChild( new domElement( 'name' ) );
		    $oAttName->appendChild( new domText( $oChild->getAttribute( 'ident' ) ) );
		    $oAttName->setAttribute( 'usage', $oChild->getAttribute( 'usage' ) );
		    
		    $oDesc = $oChild->getElementsByTagName( 'desc' )->item(0);
		    $oAttDesc = $oAtt->appendChild( new domElement( 'desc' ) );
		    $oAttDesc->appendChild( new domText( $oDesc->nodeValue ) );
		    $oDefault = $oChild->getElementsByTagName( 'defaultVal' )->item(0);
		    $oAttDef = $oAtt->appendChild( new domElement( 'defaultVal' ) );
		    $oAttDef->appendChild( new domText( $oDefault->nodeValue ) );
		  }
		else
		  {
		    $oName = $oAtt->getElementsByTagName( 'name' )->item(0);
		    if( is_object( $oName ) && $oAtt->getAttribute( 'ident' ) != $oChild->getAttribute( 'ident' ) )
		      {
			$oName->setAttribute( 'ident', $oChild->getAttribute( 'ident' )  );
			$oAltIdent = $oChild->getElementsByTagName( 'altIdent' )->item(0);
			if ( is_object( $oAltIdent ) )
			  {
			    $oAlt = $oAtt->appendChild( new domElement( 'altName' ) );
			    $oAlt->appendChild( new domText( $oAltIdent->nodeValue ) );
			  }
		      }
		    if( is_object( $oName ) && $oAtt->getAttribute( 'usage' ) != $oChild->getAttribute( 'usage' ) )
		      {
			$oName->setAttribute( 'usage', $oChild->getAttribute( 'usage' )  );
		      }
		    
		    $oDesc = $oAtt->getElementsByTagName( 'desc' )->item(0);
		    $oChildDesc = $oChild->getElementsByTagName( 'desc' )->item(0);
		    if ( is_object( $oDesc ) && is_object( $oChildDesc ) &&  $oDesc->nodeValue != $oChildDesc->nodeValue )
		      {
			$oAtt->removeChild( $oDesc );
		      }
		    if ( is_object( $oChildDesc ) )
		      {
			$oAttDesc = $oAtt->appendChild( new domElement( 'desc' ) );
			$oAttDesc->appendChild( new domText( $oChildDesc->nodeValue ) );
		      }
		    
		    
		    $oChildDefault = $oChild->getElementsByTagName( 'defaultVal' )->item(0);
		    $oDefault = $oAtt->getElementsByTagName( 'defaultVal' )->item(0);
		    if ( is_object( $oDefault ) && is_object( $oChildDefault ) && $oDefault->nodeValue != $oChildDefault->nodeValue )
		      { 
			$oAtt->removeChild( $oDefault );
		      }
		    if ( is_object( $oChildDefault ) )
		      {
			$oAttDef = $oAtt->appendChild( new domElement( 'defaultVal' ) );
			$oAttDef->appendChild( new domText( $oChildDefault->nodeValue ) );
		      }

		    $oChildDat = $oChild->getElementsByTagName( 'datatype' )->item(0);
		    $oDat = $oAtt->getElementsByTagName( 'datatype' )->item(0);
		    if ( is_object( $oDat ) && is_object( $oChildDat ) )
		      {
			$oAtt->removeChild( $oDat );
		      }
		  }

		$oInclude = $oAtt->getElementsByTagName( 'include' )->item(0);
		if( is_object( $oInclude ) )
		  {
		    $oAtt->removeChild( $oInclude );
		  }
		$oAtt->appendChild( new domElement( 'include',$oChild->getAttribute( 'mode' ) ) );

// look at datatype

		$oAttDat = $oChild->getElementsByTagName( 'datatype' )->item(0);
		if ( is_object( $oAttDat ) )
		  {
		    $oTest = $oAtt->appendChild( new domElement( 'whatever' ) );
		    $oDataType = $oAtt->appendChild( new domElement( 'datatype' ) );
		    $oDataType->setAttribute( 'minOccurs', $oAttDat->getAttribute( 'minOccurs' )  );

  		    $oDataType->setAttribute( 'maxOccurs', $oAttDat->getAttribute( 'maxOccurs' )  );

		    if ( $oAttDat->firstChild->nodeName == 'rng:text' )
		      {
			$oDataType->appendChild( new domText( 'text' ) );
		      }
	       	    elseif( $oAttDat->firstChild->nodeName == 'rng:data' )
		      {
			$theNode = $oAttDom->createElementNS( 'http://relaxng.org/ns/structure/1.0', 'rng:data' );
			$oNode = $oDataType->appendChild( $theNode );
			$oNode->setAttribute( 'type', $oAttDat->firstChild->getAttribute( 'type' ) );
		      }
		    else
		      { 
			$theNode = $oAttDom->createElementNS( 'http://relaxng.org/ns/structure/1.0', 'rng:ref' );
			$oNode = $oDataType->appendChild( $theNode );
			$oNode->setAttribute( 'name', $oAttDat->firstChild->getAttribute( 'name' ) );
		      }
		  }
	      }
	  }

	return $errResult;
      }

// getting the currentAttribute definition 
    public function getAttributeDefinition( $szAttribute, $szElement, $szModule, $szClass,&$oDef )
      {
	$this->getXPath( $oXPath );
	
	$oDef = new domDocument;
	$oRoot = $oDef->appendChild( new domElement( 'attDef' ) );
	
	$this->getAttributeDomByElementInModule( $szElement,
	$szModule, $szClass, $oAtts );

	$oXPath = new domxpath( $oAtts );
	$oAttDef = $oXPath->query( "/Element/att[child::name[node()='$szAttribute']]" )->item(0);

	$oAdd = $oRoot->appendChild( new domElement( 'added' ) );
	$oAdd->appendChild( new domText( $oAttDef->getAttribute( 'added' ) ) );

	$this->getAttributeValList( $szAttribute, $szElement,
             $szModule, $szClass, $aszList, $aszListType );
        $szValList = join( ',', $aszList );


	foreach( $oAttDef->childNodes as $oChild )
	  {
	    switch( $oChild->nodeName )
	      {
	      case 'name':
		$oName = $oRoot->appendChild( new domElement( 'attName' ) );
		$oName->appendChild( new domText( $oChild->nodeValue ) );
		$oOpt = $oRoot->appendChild( new domElement( 'optional' ) );
		$oOpt->appendChild( new domText( $oChild->getAttribute( 'usage' ) ) );
		break;
	      case 'datatype':
		$oNode = $oRoot->appendChild( new domElement( 'datatype' ) );
	        $oNode->setAttribute('minOccurs',$oChild->getAttribute( 'minOccurs'));
	        $oNode->setAttribute('maxOccurs',$oChild->getAttribute( 'maxOccurs'));
		if ( $oChild->firstChild->nodeName == 'rng:text' )
		  {
		    $oNode->appendChild( new domText( 'text' ) );
		  }
		elseif( $oChild->firstChild->nodeName == 'rng:data' )
		  {
		    $oNode->appendChild( new domText( $oChild->firstChild->getAttribute( 'type' ) ) );
		  }
		elseif( $oChild->firstChild->nodeName == 'rng:ref' )
		  {
		    $oNode->appendChild( new domText( $oChild->firstChild->getAttribute( 'name' ) ) );
		  }
		break;
	      case 'valList':
	        if ($szValList == '') {
		 foreach( $oChild->childNodes as $oChild )
		   {
		    $aszList[] = $oChild->getAttribute( 'ident' );
		   }
		   $szValList = join( ',', $aszList );	
		  }
		  if ($aszListType =='') {
		     $aszListType = $oChild->getAttribute( 'type' );
		  }
	       break;
	      default:
		$oNode = $oRoot->appendChild( new domElement( $oChild->nodeName ) );
		$oNode->appendChild( new domText( $oChild->nodeValue ) );
		break;		      
	      }
	  }
	

	$oValList = $oRoot->appendChild( new domElement( 'valList', $szValList ));
	if ($aszListType != '') {
	 $oValList->setAttribute('type',$aszListType);
	}
      }

    public function getAttributeValList( $szAttribute, $szElement,
      $szModule, $szClass, &$aszList, &$aszListType )
      {
	$aszList = array();
	$this->getXPath( $oXPath );

	if ( $szModule != '' && $szClass == '' )
	  {
	    $oValList = $oXPath->query( "//tei:schemaSpec/tei:elementSpec[@ident='{$szElement}' and @module='$szModule']/tei:attList/tei:attDef[@ident='{$szAttribute}']/tei:valList" )->item(0);

	    if ( is_object( $oValList ) )
	      {
		foreach( $oValList->childNodes as $oChild )
		  {
		    $aszList[] = $oChild->getAttribute( 'ident' );
		  }
		  $aszListType = $oValList->getAttribute( 'type' );
	      }
	  }
	elseif( $szClass == '' )
	  {
	    $oValList = $oXPath->query( "//tei:schemaSpec/tei:elementSpec[@ident='{$szElement}']/tei:attList/tei:attDef[@ident='{$szAttribute}']/tei:valList" )->item(0);

	    if ( is_object( $oValList ) )
	      {
		foreach( $oValList->childNodes as $oChild )
		  {
		    $aszList[] = $oChild->getAttribute( 'ident' );
		  }
		  $aszListType = $oValList->getAttribute( 'type' );
	      }
	    
	  }
	else
	  {

	    $oValList = $oXPath->query("//tei:schemaSpec/tei:classSpec[@module='$szModule' and @ident='{$szClass}']/tei:attList/tei:attDef[@ident='{$szAttribute}']/tei:valList" )->item(0);

	    if ( is_object( $oValList ) )
	      {
		foreach( $oValList->childNodes as $oChild )
		  {
		    $aszList[] = $oChild->getAttribute( 'ident' );
		  }
		  $aszListType = $oValList->getAttribute( 'type' );
	      }
	  }
      }



    public function getChangedAttributeClasses( &$aszClasses )
      {
	$this->getXPath( $oXPath );
        $aoClasses = $oXPath->query( "//tei:schemaSpec/tei:classSpec/@key" );

	$aszClasses = array();
	foreach( $aoClasses as $oClass )
	  {
	    $aszClasses[] = $oClass->nodeValue;
          }
      }

     public function getAttributeClassChanges( $szClass, &$oDom )
      {
	$oDom = new domDocument();
	$oClass = $oDom->appendChild( new domElement( 'class' ) );
	$oClass->setAttribute( 'ident', $szClass );

	$this->getXPath( $oXPath );
        $szModule = $oXPath->query( "//tei:schemaSpec/tei:classSpec[@ident='{$szClass}']]/@key" )->item(0)->nodeValue;

	$oClass->appendChild( new domElement( 'module', $szModule ) );
	$oAttList = $oXPath->query( "//tei:schemaSpec/tei:classSpec[@key='{$szClass}']/tei:attList" )->item(0);

	$oAttList = $oDom->importNode( $oAttList, true );
	$oClass->appendChild( $oAttList );
      }


    public function getCustomizationTitle( &$szTitle )
      {
	$this->getXPath( $oXPath );
	$szTitle = $oXPath->query( "/tei:TEI/tei:teiHeader/tei:fileDesc/tei:titleStmt/tei:title" )->item(0)->nodeValue;
      }

    public function getCustomizationAuthor( &$szAuthor )
      {
	$this->getXPath( $oXPath );
	$szAuthor = $oXPath->query( "/tei:TEI/tei:teiHeader/tei:fileDesc/tei:titleStmt/tei:author" )->item(0)->nodeValue;
      }

    public function getCustomizationFilename( &$szFilename )
      {
	$this->getXPath( $oXPath );
	$szFilename = $oXPath->query( "//tei:schemaSpec/@ident" )->item(0)->nodeValue;
      }

    public function getCustomizationPrefix( &$szPrefix )
      {
	$this->getXPath( $oXPath );
	$szPrefix = $oXPath->query( "//tei:schemaSpec/@prefix" )->item(0)->nodeValue;
      }

    public function getCustomizationLanguage( &$szLanguage )
      {
	$this->getXPath( $oXPath );
	$xmllang = $oXPath->query( "/tei:TEI/@xml:lang")->item(0)->nodeValue;
	switch( $xmllang ) 
		  {
  	case 'en'    : $szLanguage='en'; break;
	case 'en-US' : $szLanguage='en'; break;
	case 'en-GB' : $szLanguage='en'; break;
	case 'fr'    : $szLanguage='fr'; break;
	case 'de'    : $szLanguage='de'; break;
	case 'it'    : $szLanguage='it'; break;
	case 'zh-tw' : $szLanguage='zh-tw'; break;
	case 'es' :    $szLanguage='es'; break;
	case 'zh'    : $szLanguage='zh-tw'; break;
	case 'ja'    : $szLanguage='ja'; break;
	case 'ru'    : $szLanguage='ru'; break;
	case 'sw'    : $szLanguage='sw'; break;
	default: $szLanguage='en'; break;
      }
      }

    public function getOddLanguage( &$szOddLanguage )
      {
	$this->getXPath( $oXPath );
	$szOddLanguage = $oXPath->query( "/tei:TEI//tei:schemaSpec/@targetLang" )->item(0)->nodeValue;
        if ($szOddLanguage=='') { $szOddLanguage='en'; }
      }

    public function getDocLanguage( &$szDocLanguage )
      {
	$this->getXPath( $oXPath );
	$szDocLanguage = $oXPath->query( "/tei:TEI//tei:schemaSpec/@docLang" )->item(0)->nodeValue;
        if ($szDocLanguage=='') { $szDocLanguage='en'; }
      }

    public function getCustomizationDescription( &$szDesc )
      {
	$this->getXPath( $oXPath );
	$szDesc = $oXPath->query( "/tei:TEI/tei:text/tei:body/tei:p[1]" )->item(0)->nodeValue;
      }
    

    // #####################################################################
    // --- Change the Customization
    // #####################################################################

    public function addModule( $szModule, $bInclude = true )
      {
	$errResult = false;

	$this->getXPath( $oXPath );

        $oModuleRef = $oXPath->query( "//tei:schemaSpec/tei:moduleRef[@key='$szModule']" )->item(0);        
        if (! is_object( $oModuleRef ) )
          {
            $oSchema = $oXPath->query( "//tei:schemaSpec" )->item(0);
	    if ( is_object( $oSchema ) )
              {
                $theModRef = $this->createElementNS( 'http://www.tei-c.org/ns/1.0', 'moduleRef' );
                $oModuleRef = $oSchema->appendChild( $theModRef );
		$oModuleRef->setAttribute( 'key', $szModule );
              }
          }

	return $errResult;
      }

    public function removeModule( $szModule )
      {
	$errResult = false;

	$this->getXPath( $oXPath );

        $oModuleRef = $oXPath->query( "//tei:schemaSpec/tei:moduleRef[@key='$szModule']" )->item(0);
        $oSchema = $oXPath->query( "//tei:schemaSpec" )->item(0);
        $oSchema->removeChild( $oModuleRef );

	return $errResult;
      }
	

    public function addElement( $aszConfig )
      {
	if (! preg_match( '/^[\w\d]+$/', $aszConfig[ 'name' ] ) )
	  {
	    throw new falseTagnameException( '', $aszConfig[ 'name' ] );
	  }

	//check if name already exists
	$oTmpDom = new domDocument();
	if ( @$oTmpDom->loadXML( join( '', file( roma_xquery_server . 'element.xq?lang=' . $_SESSION['docLang'] . '&name=' . $aszConfig[ 'name' ] ) ) ) )
	  {
	    throw new elementExistsException( $aszConfig[ 'name' ] );
	  }
	     
	$this->getXPath( $oXPath );
        $oSchema = $oXPath->query( "//tei:schemaSpec" )->item(0);

        if ( is_object( $oSchema ) )
          {
            //check whether element already exists
            $oElementSpec = $oXPath->query( "//tei:schemaSpec/tei:elementSpec[@ident='" . $aszConfig[ 'name' ] . "']" )->item(0);

	    if ( ! is_object( $oElementSpec ) )
	      {
		$theElementSpec = $this->createElementNS( 'http://www.tei-c.org/ns/1.0', 'elementSpec' );
		$oElementSpec = $oSchema->appendChild( $theElementSpec );
		$oElementSpec->setAttribute( 'ident', $aszConfig[ 'name' ] );
		$oElementSpec->setAttribute( 'mode', ( ( $aszConfig[ 'added' ] == 'true' ) ? 'add' : 'change' ) );
	      }

	    $oDesc = $oXPath->query( "//tei:schemaSpec/tei:elementSpec[@ident='{$aszConfig[ 'name' ]}']/tei:desc" )->item(0);

	    if ( is_object( $oDesc ) )
	      {
		$oElementSpec->removeChild( $oDesc );
	      }
            $theDesc = $this->createElementNS( 'http://www.tei-c.org/ns/1.0', 'desc' );
            $oDesc = $oElementSpec->appendChild( $theDesc );
	    $oDesc->appendChild( new domText( stripslashes( $aszConfig[ 'description' ] ) ) );

	    $oClasses = $oElementSpec->getElementsByTagname( 'classes' )->item(0);
	    if ( is_object( $oClasses ) )
	      {
		$oElementSpec->removeChild( $oClasses );
	      }
            $theClasses = $this->createElementNS( 'http://www.tei-c.org/ns/1.0', 'classes' );
            $oClasses = $oElementSpec->appendChild( $theClasses );
	    
	    foreach( $aszConfig[ 'classes' ] as $szClass )
              {
		$theMemberOf = $this->createElementNS( 'http://www.tei-c.org/ns/1.0', 'memberOf' );
		$oMember = $oClasses->appendChild( $theMemberOf );
		$oMember->setAttribute( 'key', $szClass );
              }

	    $oContent = $oElementSpec->getElementsByTagname( 'content' )->item(0);
            $szDesc=$aszConfig[ 'description' ];
	    if ( is_object( $oContent ) )
	      {
		$oElementSpec->removeChild( $oContent );
	      }
	    if( $aszConfig[ 'content' ] == 'userContent' )
	      {
                $szContents=stripslashes($aszConfig[ 'contentmodel' ]);
		$oConDom = new domDocument();
		if( $oConDom->loadXML( $szContents ) )
		  {
		    $oRoot = $oConDom->documentElement;
		    
		    $oRoot = $this->importNode( $oRoot, true );
		    $oElementSpec->appendChild( $oRoot );
		  }
		else
		  {
		    throw new falseContentsException( $aszConfig[ 'contentmodel' ] );
		    $errResult = true;
		  }
	      }
	    else
	      {
		$theContent = $this->createElementNS( 'http://www.tei-c.org/ns/1.0', 'content' );
		$oContent = $oElementSpec->appendChild( $theContent );
		
		switch( $aszConfig[ 'content' ] )
		  {
		  case 'empty':
		  case 'text':
		    $oRNG = $this->createElementNS( 'http://relaxng.org/ns/structure/1.0', 'rng:' . $aszConfig[ 'content' ] );
		    $oContent->appendChild( $oRNG );
		    break; 
		  default:
		    $oRNG = $this->createElementNS( 'http://relaxng.org/ns/structure/1.0', 'rng:ref' );
		    $oRef = $oContent->appendChild( $oRNG );
		    $oRef->setAttribute( 'name', $aszConfig[ 'content' ] );
		    break;
		  }
	      }

          }
      }

    public function removeElementFromModule( $szElement, $szModule )
      {
	$this->getXPath( $oXPath );
        $oSchema = $oXPath->query( "//tei:schemaSpec" )->item(0);
        $oModule = $oXPath->query( "//tei:schemaSpec//tei:moduleRef[@key='$szModule']" )->item(0);
	$oElementSpec = $oXPath->query(
      "//tei:schemaSpec/tei:elementSpec[@module='$szModule' and @ident='$szElement']" )->item(0);
	
        if ( ! is_object( $oModule ) ) 
	  {
            $this->addModule( $szModule, false );
          }

	if (! is_object( $oElementSpec ) )
	  {
	    $theElementSpec = $this->createElementNS( 'http://www.tei-c.org/ns/1.0', 'elementSpec' );
 	    $oElementSpec = $oSchema->appendChild( $theElementSpec );
	    $oElementSpec->setAttribute( 'module', $szModule );
	    $oElementSpec->setAttribute( 'ident', $szElement );
	    $oElementSpec->setAttribute( 'mode', 'delete' );
	  }
	else
	  {
	    $aoChildren = $oElementSpec->childNodes;
	    foreach( $aoChildren as $oChild )
	      {
		$oElementSpec->removeChild( $oChild );
	      }
	    $oElementSpec->setAttribute( 'mode', 'delete' );
	  }
	
      }

    public function includeElementInModule( $szElement, $szModule )
      {
	$this->getXPath( $oXPath );
        $oSchema = $oXPath->query( "//tei:schemaSpec" )->item(0);
        $oModule = $oXPath->query( "//tei:schemaSpec/tei:moduleRef[@key='$szModule']" )->item(0);
	$oElementSpec = $oXPath->query("//tei:schemaSpec/tei:elementSpec[@module='$szModule' and @ident='$szElement']" )->item(0);

        if ( ! is_object( $oModule ) ) 
	  {
            $this->addModule( $szModule, false );
          }
	
	if ( is_object( $oElementSpec ) )
	  {
	    $oSchema->removeChild( $oElementSpec );
	  }
      }

    public function changeElementNameInModule( $szOldName, $szNewName, $szModule )
      {
	$errResult = false;

	if ( preg_match( '/^[\w\d]+$/', $szNewName ) )
	  {
	    $this->getXPath( $oXPath );
            $oSchema = $oXPath->query( "//tei:schemaSpec" )->item(0);
            $oModule = $oXPath->query( "//tei:schemaSpec/tei:moduleRef[@key='$szModule']" )->item(0);
	    $oElementSpec = $oXPath->query( "//tei:schemaSpec/tei:elementSpec[@ident='$szOldName']" )->item(0);
	
            if ( ! is_object( $oModule ) ) 
	      {
                $this->addModule( $szModule, false );
              }

  	    if (! is_object( $oElementSpec ) )
 	      {
                $theElementSpec = $this->createElementNS( 'http://www.tei-c.org/ns/1.0', 'elementSpec' );
 	        $oElementSpec = $oSchema->appendChild( $theElementSpec );
	        $oElementSpec->setAttribute( 'module', $szModule );
	        $oElementSpec->setAttribute( 'ident', $szOldName );
	        $oElementSpec->setAttribute( 'mode', 'change' );
		
                $theAltIdent = $this->createElementNS( 'http://www.tei-c.org/ns/1.0', 'altIdent' );
   	        $oAltIdent = $oElementSpec->appendChild( $theAltIdent );
	        $oAltIdent->appendChild( new domText( $szNewName ) );
	      }
  	    else
  	      {
	        $oElementSpec->setAttribute( 'mode', 'change' );

		$oAltIdent = $oXPath->query( "//tei:schemaSpec/tei:elementSpec[@ident='$szOldName']/tei:altIdent" )->item(0);
		
		if ( is_object( $oAltIdent ) )
		  {
		    $oElementSpec->removeChild( $oAltIdent );
		  }
	      
                $theAltIdent = $this->createElementNS( 'http://www.tei-c.org/ns/1.0', 'altIdent' );
	        $oAltIdent = $oElementSpec->appendChild( $theAltIdent );
  	        $oAltIdent->appendChild( new domText( $szNewName ) );
 	     }
          }
	else
	  {
	    throw new falseTagnameException( $szOldName, $szNewName );
	    $errResult = $szOldName;
	  }
	
	return $errResult;
      }

    public function changeElementDescInModule( $szName, $szModule, $szDescription )
      {
	$this->getXPath( $oXPath );
        $oSchema = $oXPath->query( "//tei:schemaSpec" )->item(0);
	$oModule = $oXPath->query( "//tei:schemaSpec/tei:moduleRef[@key='$szModule']" )->item(0);
	$oElementSpec = $oXPath->query("//tei:schemaSpec/tei:elementSpec[@ident='$szName']" )->item(0);
	
	if ( ! is_object( $oModule ) ) 
	  {
	    $this->addModule( $szModule, false );
	  }
	
	if (! is_object( $oElementSpec ) )
	  {
	    $theElementSpec = $this->createElementNS( 'http://www.tei-c.org/ns/1.0', 'elementSpec' );
	    $oElementSpec = $oSchema->appendChild( $theElementSpec );
	    $oElementSpec->setAttribute( 'ident', $szName );
	    $oElementSpec->setAttribute( 'module', $szModule );
	    $oElementSpec->setAttribute( 'mode', 'change' );
	    
	    $theDesc = $this->createElementNS( 'http://www.tei-c.org/ns/1.0', 'desc' );
	    $oDesc = $oElementSpec->appendChild( $theDesc );
	    $oDesc->appendChild( new domText( $szDescription ) );
	  }
	else
	  {
	    $oElementSpec->setAttribute( 'mode', 'change' );

	    $oDesc = $oXPath->query(
	    "//tei:schemaSpec/tei:elementSpec[@ident='$szName']/tei:desc")->item(0);
	    
	    if ( is_object( $oDesc ) )
	      {
	      if ($oDesc != $theDesc) 
	       {
		$oElementSpec->removeChild( $oDesc );
                $theDesc = $this->createElementNS( 'http://www.tei-c.org/ns/1.0', 'desc' );
	        $oDesc = $oElementSpec->appendChild( $theDesc );
	        $oDesc->appendChild( new domText( stripslashes($szDescription) ) );
	      }
	  }
      }
    }

    public function changeElementContentsInModule( $szElement, $szModule, $szContents )
      {
	$errResult = false;

	$szContents = preg_replace( '%\\\\"%', '"', $szContents );

	$this->getXPath( $oXPath );
        $oSchema = $oXPath->query( "//tei:schemaSpec" )->item(0);
	$oModule = $oXPath->query( "//tei:schemaSpec/tei:moduleRef[@key='$szModule']" )->item(0);
	$oElementSpec = $oXPath->query("//tei:schemaSpec/tei:elementSpec[@ident='$szElement']" )->item(0);
	$oContent = $oXPath->query("//tei:schemaSpec/tei:elementSpec[@ident='$szElement']/tei:content" )->item(0);
	if ( ! is_object( $oModule ) ) 
	  {
	    $this->addModule( $szModule, false );
	    }	    
	if (! is_object( $oElementSpec ) )
	  {
       	    $theElementSpec = $this->createElementNS( 'http://www.tei-c.org/ns/1.0', 'elementSpec' );
	    $oElementSpec = $oSchema->appendChild( $theElementSpec );
	    $oElementSpec->setAttribute( 'ident', $szElement );
	    $oElementSpec->setAttribute( 'module', $szModule );
	    $oElementSpec->setAttribute( 'mode', 'change' );
	  }


	if ( is_object( $oContent ) )
          {
	    $oElementSpec->removeChild( $oContent );
          }

	$oElementSpec->setAttribute( 'mode', 'change' );
	
	//Load Content
	$oConDom = new domDocument();
	if( $oConDom->loadXML( $szContents ) )
	  {
	    $oRoot = $oConDom->documentElement;
	    
	    $oRoot = $this->importNode( $oRoot, true );
	    $oElementSpec->appendChild( $oRoot );
	  }
	else
	  {
	    throw new falseContentsException( $szContents );
	    $errResult = true;
	  }

	return $errResult;
      }

 
    public function replaceElementsClassesInModule( $szElement, $szModule, $aszClasses )
      {
	$this->getXPath( $oXPath );
        $oSchema = $oXPath->query( "//tei:schemaSpec" )->item(0);
	$oModule = $oXPath->query( "//tei:schemaSpec/tei:moduleRef[@key='{$szModule}']" )->item(0);
	$oElementSpec = $oXPath->query("//tei:schemaSpec/tei:elementSpec[@module='{$szModule}' and @ident='{$szElement}']" )->item(0);
	if ( ! is_object( $oModule ) ) 
	  {
	    $this->addModule( $szModule, false );
	  }

	if (! is_object( $oElementSpec ) )
	  {
       	    $theElementSpec = $this->createElementNS( 'http://www.tei-c.org/ns/1.0', 'elementSpec' );
	    $oElementSpec = $oSchema->appendChild( $theElementSpec );
	    $oElementSpec->setAttribute( 'ident', $szElement );
	    $oElementSpec->setAttribute( 'module', $szModule );
	    $oElementSpec->setAttribute( 'mode', 'change' );
	  }

	//check whether there are any classes yet
	$oClasses = $oXPath->query(
	"//tei:schemaSpec/tei:elementSpec[@module='{$szModule}' and @ident='{$szElement}']/tei:classes" )->item(0);
	
	if ( is_object( $oClasses ) )
	  {
	    $oElementSpec->removeChild( $oClasses );
	  }

	$theClasses = $this->createElementNS( 'http://www.tei-c.org/ns/1.0', 'classes' );
	$oClasses = $oElementSpec->appendChild( $theClasses );
	
	foreach ( $aszClasses as $szClass )
	  {
	    $theMemberOf = $this->createElementNS( 'http://www.tei-c.org/ns/1.0', 'memberOf' );
	    $oMember = $oClasses->appendChild( $theMemberOf );
	    $oMember->setAttribute( 'key', $szClass );
	  }

      }  

    public function addAttribute( $aszConfig )
      {
	$errResult = false;
	if (! preg_match( '/^[\w\d:-]+$/', $aszConfig[ 'name' ] ) )
	  {
	    $errResult = true;
	    throw new falseTagnameException( '', $aszConfig[ 'name' ] );
	  }
	  // are we an element or a class
	if ( $aszConfig[ 'class' ] == '' )
	 {	$current=$aszConfig[ 'element' ] ;}
	else
	 {	$current=$aszConfig[ 'class' ] ; }

	//check if name already exists
	$this->getXPath( $oXPath );
        $oSchema = $oXPath->query( "//tei:schemaSpec" )->item(0);
	if ( ! $errResult )
	  {
	    $this->getXPath( $oXPath );

	    if ( $aszConfig[ 'module' ] != '' && $aszConfig[ 'class' ] == '' )
	      {
		$oModule = $oXPath->query( "//tei:schemaSpec/tei:moduleRef[@key='{$aszConfig[ 'module' ]}']" )->item(0);

		$oElementSpec = $oXPath->query( "//tei:schemaSpec/tei:elementSpec[@ident='{$aszConfig[ 'element' ]}']" )->item(0);

		if ( ! is_object( $oModule ) ) 
		  {
		    $this->addModule( $aszConfig[ 'module' ], false );
		  }
		
		if (! is_object( $oElementSpec ) )
		  {
		    $theElementSpec = $this->createElementNS( 'http://www.tei-c.org/ns/1.0', 'elementSpec' );
		    $oElementSpec = $oSchema->appendChild(  $theElementSpec );
		    $oElementSpec->setAttribute( 'ident', $aszConfig['element' ] );
		    $oElementSpec->setAttribute( 'module',  $aszConfig['module'] );
		  }
		$oElementSpec->setAttribute( 'mode', 'change' );
	    
		$oAttList = $oXPath->query( "//tei:schemaSpec/tei:elementSpec[@ident='{$aszConfig[ 'element' ]}']/tei:attList" )->item(0);
	    
		if (! is_object( $oAttList ) )
		  {
		    $theAttList = $this->createElementNS( 'http://www.tei-c.org/ns/1.0', 'attList' );
		    $oAttList = $oElementSpec->appendChild( $theAttList );
		  }

		$oAttDef = $oXPath->query( "//tei:schemaSpec/tei:elementSpec[@ident='{$aszConfig[ 'element' ]}']/tei:attList/tei:attDef[@ident='{$aszConfig[ 'name' ]}']" )->item(0);
	      }
	    elseif( $aszConfig[ 'class' ] == '' )
	      {
		$oElementSpec = $oXPath->query( "//tei:schemaSpec/tei:elementSpec[@ident='" . $aszConfig[ 'element' ] . "']" )->item(0);

		$oAttList = $oXPath->query( "//tei:schemaSpec/tei:elementSpec[@ident='{$aszConfig[ 'element' ]}']/tei:attList" )->item(0);
	    
		if (! is_object( $oAttList ) )
		  {
		    $theAttList = $this->createElementNS( 'http://www.tei-c.org/ns/1.0', 'attList' );
		    $oAttList = $oElementSpec->appendChild( $theAttList );
		  }

		$oAttDef = $oXPath->query( "//tei:schemaSpec/tei:elementSpec[@ident='{$aszConfig[ 'element' ]}']/tei:attList/tei:attDef[@ident='{$aszConfig[ 'name' ]}']" )->item(0);

	      }
	    else
	      {
		$oModule = $oXPath->query( "//tei:schemaSpec/tei:moduleRef[@key='{$aszConfig[ 'module' ]}']" )->item(0);
		$oClassSpec = $oXPath->query( "//tei:schemaSpec/tei:classSpec[@ident='{$aszConfig[ 'class' ]}']" )->item(0);
		
		if ( ! is_object( $oModule ) ) 
		  {
		    $this->addModule( $aszConfig[ 'module' ], false );
		  }
		
		if (! is_object( $oClassSpec ) )
		  {
		    $theClassSpec = $this->createElementNS( 'http://www.tei-c.org/ns/1.0', 'classSpec' );
		    $oClassSpec = $oSchema->appendChild( $theClassSpec );
		    $oClassSpec->setAttribute( 'ident', $aszConfig[ 'class' ] );
		    $oClassSpec->setAttribute( 'type', 'atts' );
		    $oClassSpec->setAttribute( 'mode', 'change' );
		    $oClassSpec->setAttribute( 'module', $aszConfig[ 'module' ]  );
		  }
		
		
		$oAttList = $oXPath->query( "//tei:schemaSpec/tei:classSpec[@ident='{$aszConfig[ 'class' ]}']/tei:attList" )->item(0);
		
		if (! is_object( $oAttList ) )
		  {
		    $theAttList = $this->createElementNS( 'http://www.tei-c.org/ns/1.0', 'attList' );
		    $oAttList = $oClassSpec->appendChild( $theAttList );
		  }

		$oAttDef = $oXPath->query( "//tei:schemaSpec/tei:classSpec[@ident='{$aszConfig[ 'class' ]}']/tei:attList/tei:attDef[@ident='{$aszConfig[ 'name' ]}']" )->item(0);
	      }

	    //attdef
	    if (! is_object( $oAttDef ) )
	      {
		$theAttDef = $this->createElementNS( 'http://www.tei-c.org/ns/1.0', 'attDef' );
		$oAttDef = $oAttList->appendChild( $theAttDef );
		$oAttDef->setAttribute( 'ident', $aszConfig[ 'name' ] );

		if ( $aszConfig[ 'added' ] == 'true' )
		  {
		    $oAttDef->setAttribute( 'mode', 'add' );
		  }
		else
		  {
		    $oAttDef->setAttribute( 'mode', 'change' );
		  }
	      }

	    //optional
	    
	    if ($aszConfig[ 'optional' ] == 'no')
	    	{
		    $oAttDef->setAttribute( 'usage', 'req' );
		    }
	    //desc
	    $oDesc = $oAttDef->getElementsByTagname( 'desc' )->item(0);
	    if ( is_object( $oDesc ) )
	      {
		$oAttDef->removeChild( $oDesc );
	      }
	    $theDesc = $this->createElementNS( 'http://www.tei-c.org/ns/1.0', 'desc' );
	    $oDesc = $oAttDef->appendChild( $theDesc );
	    $oDesc->appendChild( new domText( stripslashes($aszConfig[ 'description' ]) ) );

	    //content
	    $oContent = $oAttDef->getElementsByTagname( 'datatype' )->item(0);
	    if ( is_object( $oContent ) )
	      {
		$oAttDef->removeChild( $oContent );
	      }
	    $theContent = $this->createElementNS( 'http://www.tei-c.org/ns/1.0', 'datatype' );
	    $oContent = $oAttDef->appendChild( $theContent );

	    $oContent->setAttribute('minOccurs', $aszConfig['minOccurs']);
	    $oContent->setAttribute('maxOccurs', $aszConfig['maxOccurs']);
	    switch ( $aszConfig[ 'content' ] )
	      {
	      case 'text':
		$oRNG = $this->createElementNS( 'http://relaxng.org/ns/structure/1.0', 'rng:' . $aszConfig[ 'content' ] );
		$oContent->appendChild( $oRNG );
		break; 
	      default:
		    $oRNG = $this->createElementNS( 'http://relaxng.org/ns/structure/1.0', 'rng:ref' );
		    $oRef = $oContent->appendChild( $oRNG );
		    $oRef->setAttribute( 'name', $aszConfig[ 'content' ] );
		    break;
	      }

	      	    
	    //default
	    if ($aszConfig[ 'defaultValue' ] != '') {
	     $oDefault = $oAttDef->getElementsByTagname( 'defaultVal' )->item(0);
  	     if ( is_object( $oDefault ) )
	      {
		$oAttDef->removeChild( $oDefault );
	      }
	     $theDefault = $this->createElementNS( 'http://www.tei-c.org/ns/1.0', 'defaultVal' );
	     $oDefault = $oAttDef->appendChild( $theDefault );
	     $oDefault->appendChild( new domText( $aszConfig[ 'defaultValue' ] ) );
	    }
	    //valList
	    if ($aszConfig[ 'valList' ] != '') {
	     $oValList = $oAttDef->getElementsByTagname( 'valList' )->item(0);
	     if ( is_object( $oValList ) )
	      {
		$oAttDef->removeChild( $oValList );
	      }
	     $aszValList = explode( ',', $aszConfig[ 'valList' ] );
	     $theValList = $this->createElementNS( 'http://www.tei-c.org/ns/1.0', 'valList' );
	     $oValList = $oAttDef->appendChild( $theValList );
	     if ( $aszConfig[ 'closed' ] == 'true' )
	      {
		$oValList->setAttribute( 'type', 'closed' );
	      }
	     else
	      {
		$oValList->setAttribute( 'type', 'open' );
	      }
    	     if ( $aszConfig[ 'added' ] != 'true' )
	     {
		 $oValList->setAttribute( 'mode', 'replace' );	
		 }
	     if ( is_array( $aszValList ) )
	      {
		foreach( $aszValList as $szValue )
		  {
		    chop( $szValue );
		    if ( $szValue != '' )
		      {
			$theValItem = $this->createElementNS( 'http://www.tei-c.org/ns/1.0', 'valItem' );
			$oValItem = $oValList->appendChild( $theValItem );
			$oValItem->setAttribute( 'ident', $szValue );
		      }
		  }
	        }
	     }
	  }
	return $errResult;

      }

    public function includeAttributeInElement( $szAttribute, $szClass, $szModule, $szElement )
      {
	$this->getXPath( $oXPath );
        $oSchema = $oXPath->query( "//tei:schemaSpec" )->item(0);

	if ( $szModule != '' && $szClass == '')
	  {
	    $oModule = $oXPath->query( "//tei:schemaSpec/tei:moduleRef[@key='{$szModule}']" )->item(0);
	    $oElementSpec = $oXPath->query(
	  "//tei:schemaSpec/tei:elementSpec[@module='{$szModule}' and @ident='{$szElement}']" )->item(0);
	    
	    if ( ! is_object( $oModule ) ) 
	      {
		$this->addModule( $szModule, false );
	      }
	    
	    if (! is_object( $oElementSpec ) )
	      {
		$theElementSpec = $this->createElementNS( 'http://www.tei-c.org/ns/1.0', 'elementSpec' );
		$oElementSpec = $oSchema->appendChild( $theElementSpec );
		$oElementSpec->setAttribute( 'ident',  $szElement );
		$oElementSpec->setAttribute( 'module', $szModule );
		$oElementSpec->setAttribute( 'mode',   'change' );
	      }
	    
	    $oAttList = $oXPath->query(
	    "//tei:schemaSpec/tei:elementSpec[@module='{$szModule}' and @ident='{$szElement}']/tei:attList" )->item(0);
	    
	    if (! is_object( $oAttList ) )
	      {
		$theAttList = $this->createElementNS( 'http://www.tei-c.org/ns/1.0', 'attList' );
		$oAttList = $oElementSpec->appendChild( $theAttList );
	      }
	    
	    $oAttDef = $oXPath->query( "//tei:schemaSpec/tei:elementSpec[@ident='{$szElement}']/tei:attList/tei:attDef[@ident='{$szAttribute}']" )->item(0);
	  }
	elseif ( $szModule == '' && $szClass == '' )
	  {
	    $oElementSpec = $oXPath->query( "//tei:schemaSpec/tei:elementSpec[@ident='{$szElement}']" )->item(0);
	    
	    $oAttList = $oXPath->query( "//tei:schemaSpec/tei:elementSpec[@ident='{$szElement}']/tei:attList" )->item(0);
	    
	    if (! is_object( $oAttList ) )
	      {
		$theAttList = $this->createElementNS( 'http://www.tei-c.org/ns/1.0', 'attList' );
		$oAttList = $oElementSpec->appendChild( $theAttList );
	      }
	    
	    $oAttDef = $oXPath->query( "//tei:schemaSpec/tei:elementSpec[@ident='{$szElement}']/tei:attList/tei:attDef[@ident='{$szAttribute}']" )->item(0);
	  }
	else
	  {
	    $oModule = $oXPath->query( "//tei:schemaSpec/tei:moduleRef[@key='{$szModule}']" )->item(0);
	    $oClassSpec = $oXPath->query( "//tei:schemaSpec/tei:classSpec[@ident='{$szClass}']" )->item(0);
	    
	    if ( ! is_object( $oModule ) ) 
	      {
		$this->addModule( $szModule, false );
	      }
	    
	    if (! is_object( $oClassSpec ) )
	      {
		$theClassSpec = $this->createElementNS( 'http://www.tei-c.org/ns/1.0', 'classSpec' );
		$oClassSpec = $oSchema->appendChild( $theClassSpec );
		$oClassSpec->setAttribute( 'ident', $szClass );
		$oClassSpec->setAttribute( 'mode', 'change' );
		$oClassSpec->setAttribute( 'type', 'atts' );
		$oClassSpec->setAttribute( 'module', $szModule );
	      }

	
	    $oAttList = $oXPath->query( "//tei:schemaSpec/tei:classSpec[@ident='{$szClass}']/tei:attList" )->item(0);
	    
	    if (! is_object( $oAttList ) )
	      {
		$theAttList = $this->createElementNS( 'http://www.tei-c.org/ns/1.0', 'attList' );
		$oAttList = $oClassSpec->appendChild( $theAttList );
	      }
	    
	    $oAttDef = $oXPath->query( "//tei:schemaSpec/tei:classSpec[@ident='{$szClass}']/tei:attList/tei:attDef[@ident='{$szAttribute}']" )->item(0);
	  }


	if ( ! is_object( $oAttDef ) )
	  {
	    //attdef
	    $theAttDef = $this->createElementNS( 'http://www.tei-c.org/ns/1.0', 'attDef' );
	    $oAttDef = $oAttList->appendChild( $theAttDef );
	    $oAttDef->setAttribute( 'ident', $szAttribute );
	  }

	$oAttDef->setAttribute( 'mode', 'change' );
      }
    


    public function excludeAttributeInElement( $szAttribute, $szClass, $szModule, $szElement )
      {
	$this->getXPath( $oXPath );
        $oSchema = $oXPath->query( "//tei:schemaSpec" )->item(0);

	if ( $szModule != '' && $szClass == '' )
	  {
	    $oModule = $oXPath->query( "//tei:schemaSpec/tei:moduleRef[@key='{$szModule}']" )->item(0);
	    $oElementSpec = $oXPath->query( "//tei:schemaSpec/tei:elementSpec[@ident='{$szElement}']" )->item(0);
	    
	    if ( ! is_object( $oModule ) ) 
	      {
		$this->addModule( $szModule, false );
	      }
	    
	    if (! is_object( $oElementSpec ) )
	      {
		$theElementSpec = $this->createElementNS( 'http://www.tei-c.org/ns/1.0', 'elementSpec' );
		$oElementSpec = $oSchema->appendChild( $theElementSpec );
		$oElementSpec->setAttribute( 'ident', $szElement );
		$oElementSpec->setAttribute( 'mode', 'change' );
	      }
	
	    $oAttList = $oXPath->query( "//tei:schemaSpec/tei:elementSpec[@ident='{$szElement}']/tei:attList" )->item(0);
	    
	    if (! is_object( $oAttList ) )
	      {
		$theAttList = $this->createElementNS( 'http://www.tei-c.org/ns/1.0', 'attList' );
		$oAttList = $oElementSpec->appendChild( $theAttList );
	      }
	    
	    $oAttDef = $oXPath->query( "//tei:schemaSpec/tei:elementSpec[@ident='{$szElement}']/tei:attList/tei:attDef[@ident='{$szAttribute}']" )->item(0);
	  }
	elseif ( $szModule == '' && $szClass == '' )
	  {
	    $oElementSpec = $oXPath->query( "//tei:schemaSpec/tei:elementSpec[@ident='" . $szElement . "']" )->item(0);
	    
	    $oAttList = $oXPath->query( "//tei:schemaSpec/tei:elementSpec[@ident='{$szElement}']/tei:attList" )->item(0);
	    
	    if (! is_object( $oAttList ) )
	      {
		$theAttList = $this->createElementNS( 'http://www.tei-c.org/ns/1.0', 'attList' );
		$oAttList = $oElementSpec->appendChild( $theAttList );
	      }
	    
	    $oAttDef = $oXPath->query( "//tei:schemaSpec/tei:elementSpec[@ident='{$szElement}']/tei:attList/tei:attDef[@ident='{$szAttribute}']" )->item(0);
	  }
	else
	  {
	    $oModule = $oXPath->query( "//tei:schemaSpec/tei:moduleRef[@key='{$szModule}']" )->item(0);
	    $oClassSpec = $oXPath->query( "//tei:schemaSpec/tei:classSpec[@ident='{$szClass}']" )->item(0);
	    
	    if ( ! is_object( $oModule ) ) 
	      {
		$this->addModule( $szModule, false );
	      }
	    
	    if (! is_object( $oClassSpec ) )
	      {
		$theClassSpec = $this->createElementNS( 'http://www.tei-c.org/ns/1.0', 'classSpec' );
		$oClassSpec = $oSchema->appendChild( $theClassSpec );
		$oClassSpec->setAttribute( 'ident', $szClass );
		$oClassSpec->setAttribute( 'module', $szModule );
		$oClassSpec->setAttribute( 'mode', 'change' );
		$oClassSpec->setAttribute( 'type', 'atts' );
	      }

	
	    $oAttList = $oXPath->query( "//tei:schemaSpec/tei:classSpec[@ident='{$szClass}']/tei:attList" )->item(0);
	    
	    if (! is_object( $oAttList ) )
	      {
		$theAttList = $this->createElementNS( 'http://www.tei-c.org/ns/1.0', 'attList' );
		$oAttList = $oClassSpec->appendChild( $theAttList );
	      }
	    
	    $oAttDef = $oXPath->query( "//tei:schemaSpec/tei:classSpec[@ident='{$szClass}']/tei:attList/tei:attDef[@ident='{$szAttribute}']" )->item(0);
	  }

	if ( ! is_object( $oAttDef ) )
	  {
	    //attdef
	    $theAttDef = $this->createElementNS( 'http://www.tei-c.org/ns/1.0', 'attDef' );
	    $oAttDef = $oAttList->appendChild( $theAttDef );
	    $oAttDef->setAttribute( 'ident', $szAttribute );
	  }

	$oAttDef->setAttribute( 'mode', 'delete' );
      }

    public function changeAttributesName( $szOldName, $szNewName, $szClass, $szModule, $szElement )
      {
	$errResult = false;

	if ( preg_match( '/^[:\w\d]+$/', $szNewName ) )
	  {
	    $this->getXPath( $oXPath );
	    $oSchema = $oXPath->query( "//tei:schemaSpec" )->item(0);	    
	    if ( $szModule != '' && $szClass == '' )
	      {
		$oSchema = $oXPath->query("//tei:schemaSpec")->item(0);
		$oModuleRef = $oXPath->query( "//tei:schemaSpec/tei:moduleRef[@key='{$szModule}']" )->item(0);
		$oElementSpec = $oXPath->query(
	      "//tei:schemaSpec/tei:elementSpec[@module='{$szModule}' and @ident='{$szElement}']" )->item(0);
	    
		if ( ! is_object( $oModuleRef ) ) 
		  {
		    $this->addModule( $szModule, false );
		  }
		
		if (! is_object( $oElementSpec ) )
		  {
		    $theElementSpec = $this->createElementNS( 'http://www.tei-c.org/ns/1.0', 'elementSpec' );
		    $oElementSpec = $oSchema->appendChild( $theElementSpec );
		    $oElementSpec->setAttribute( 'ident', $szElement  );
		    $oElementSpec->setAttribute( 'module', $szModule );
		    $oElementSpec->setAttribute( 'mode', 'change' );
		  }
		
		$oAttList = $oXPath->query("//tei:schemaSpec/tei:elementSpec[@module='{$szModule}' and @ident='{$szElement}']/tei:attList" )->item(0);
	    
		if (! is_object( $oAttList ) )
		  {
		    $theAttList = $this->createElementNS( 'http://www.tei-c.org/ns/1.0', 'attList' );
		    $oAttList = $oElementSpec->appendChild( $theAttList );
		  }
		
		$oAttDef = $oXPath->query( "//tei:schemaSpec/tei:elementSpec[@ident='{$szElement}']/tei:attList/tei:attDef[@ident='$szOldName']" )->item(0);
	      }
	    elseif ( $szClass != '' )
	      {
		$oModule = $oXPath->query( "//tei:schemaSpec/tei:moduleRef[@key='{$szModule}']" )->item(0);
		$oClassSpec = $oXPath->query( "//tei:schemaSpec/tei:classSpec[@ident='{$szClass}']" )->item(0);
		
		if ( ! is_object( $oModule ) ) 
		  {
		    $this->addModule( $szModule, false );
		  }
		
		if (! is_object( $oClassSpec ) )
		  {
		    $theClassSpec = $this->createElementNS( 'http://www.tei-c.org/ns/1.0', 'classSpec' );
		    $oClassSpec = $oSchema->appendChild( $theClassSpec );
		    $oClassSpec->setAttribute( 'ident', $szClass );
		    $oClassSpec->setAttribute( 'type', 'atts' );
		    $oClassSpec->setAttribute( 'module', $szModule );
		    $oClassSpec->setAttribute( 'mode', 'change' );
		  }
		
		
		$oAttList = $oXPath->query(
		"//tei:schemaSpec/tei:classSpec[@module='{$szModule}' and @ident='{$szClass}']/tei:attList" )->item(0);
		
		if (! is_object( $oAttList ) )
		  {
		    $theAttList = $this->createElementNS( 'http://www.tei-c.org/ns/1.0', 'attList' );
		    $oAttList = $oClassSpec->appendChild( $theAttList );
		  }
		
		$oAttDef = $oXPath->query(
	      "//tei:schemaSpec/tei:classSpec[@module='{$szModule}' and @ident='{$szClass}']/tei:attList/tei:attDef[@ident='$szOldName']" )->item(0);
	      }
	    
	    if ( ! is_object( $oAttDef ) )
	      {
		//attdef
		$theAttDef = $this->createElementNS( 'http://www.tei-c.org/ns/1.0', 'attDef' );
		$oAttDef = $oAttList->appendChild( $theAttDef );
		$oAttDef->setAttribute( 'ident', $szOldName );
	      }
	    
	    $oAltIdent = $oAttDef->getElementsByTagName( 'altIdent' )->item(0);
	    if( is_object( $oAltIdent ) )
	      {
		$oAttDef->removeChild( $oAltIdent );
	      }
	    
	    $theAltIdent = $this->createElementNS( 'http://www.tei-c.org/ns/1.0', 'altIdent' );
	    $oAltIdent = $oAttDef->appendChild( $theAltIdent );
	    $oAltIdent->appendChild( new domText( $szNewName ) );
	  }
	else
	  {
	    throw new falseTagnameException( $szOldName, $szNewName );
	    $errResult = true;
	  }

	return $errResult;
      }

    public function deleteAttribute( $szAttribute, $szClass, $szModule, $szElement )
      {
	$this->getXPath( $oXPath );

	if ( $szModule != '' && $szClass == '' )
	  {
	    $oAttDef = $oXPath->query(
	  "//tei:schemaSpec/tei:elementSpec[@module='{$szModule}' and @ident='{$szElement}']/tei:attList/tei:attDef[@ident='{$szAttribute}']" )->item(0);
	  }
	elseif( $szClass == '' )
	  {
	    $oAttDef = $oXPath->query( "//tei:schemaSpec/tei:elementSpec[@ident='{$szElement}']/tei:attList/tei:attDef[@ident='{$szAttribute}']" )->item(0);
	  }
	else
	  {
	    $oAttDef = $oXPath->query(
	  "//tei:schemaSpec/tei:classSpec[@module='{$szModule}' and @ident='{$szClass}']/tei:attList/tei:attDef[@ident='{$szAttribute}']" )->item(0);
	  }

	$oAttDef->parentNode->removeChild( $oAttDef );
      }  


    public function includeAddedElement( $szElement )
      {
	$this->getXPath( $oXPath );
	
	$oElementSpec = $oXPath->query( "//tei:schemaSpec/tei:elementSpec[@ident='{$szElement}']" )->item(0);

	$oElementSpec->setAttribute( 'mode', 'change' );
      }

    public function excludeAddedElement( $szElement )
      {
	$this->getXPath( $oXPath );
	
	$oElementSpec = $oXPath->query( "//tei:schemaSpec/tei:elementSpec[@ident='{$szElement}']" )->item(0);

	$oElementSpec->setAttribute( 'mode', 'delete' );
      }

    public function deleteAddedElement( $szElement )
      {
	$errResult = false;

	$this->getXPath( $oXPath );
	$oElementSpec = $oXPath->query( "//tei:schemaSpec/tei:elementSpec[@ident='{$szElement}']" )->item(0);
	if ( is_object( $oElementSpec ) )
	  {
	    $oElementSpec->parentNode->removeChild( $oElementSpec );
	  }
	else
	  {
	    $errResult = true;
	  }

	return $errResult;
      }


    public function setCustomizationTitle( $szTitle )
      {
	$this->getXPath( $oXPath );
	$oTitle = $oXPath->query( "/tei:TEI/tei:teiHeader/tei:fileDesc/tei:titleStmt/tei:title" )->item(0);
	if ( $oTitle->hasChildNodes() )
	  $oTitle->removeChild( $oTitle->firstChild );
	$oTitle->appendChild( new domText ( $szTitle ) );
      }

    public function setCustomizationDescription( $szDescription )
      {
	$this->getXPath( $oXPath );
	$oP = $oXPath->query( "/tei:TEI/tei:text/tei:body/tei:p" )->item(0);
	if ( $oP->hasChildNodes() )
	  $oP->removeChild( $oP->firstChild );
	$oP->appendChild( new domText ( stripslashes($szDescription) ) );
      }

    public function setCustomizationAuthor( $szAuthor )
      {
	$this->getXPath( $oXPath );
	$oAuthor = $oXPath->query("/tei:TEI/tei:teiHeader/tei:fileDesc/tei:titleStmt/tei:author")->item(0);
	if ( is_object( $oAuthor ) ) {
   	 if ( $oAuthor->hasChildNodes() )
	   $oAuthor->removeChild( $oAuthor->firstChild );
	 }
	else
	{
 	 $oTitleS = $oXPath->query("/tei:TEI/tei:teiHeader/tei:fileDesc/tei:titleStmt")->item(0);
  	 $oAuthor = $oTitleS->appendChild( new domElement( 'author' ) );
	}
        $oAuthor->appendChild( new domText ( $szAuthor ) );
      }

    public function setCustomizationFilename( $szFilename )
      {
	$this->getXPath( $oXPath );
	$oTEI = $oXPath->query( "//tei:schemaSpec[1]" )->item(0);
	$oTEI->setAttribute( 'ident', $szFilename );
      }

    public function setCustomizationPrefix( $szPrefix )
      {
	$this->getXPath( $oXPath );
	$oTEI = $oXPath->query( "//tei:schemaSpec[1]" )->item(0);
	$oTEI->setAttribute( 'prefix', $szPrefix );
      }

    public function setCustomizationLanguage( $szLanguage )
      {
	$this->getXPath( $oXPath );
	$oTEI = $oXPath->query( "/tei:TEI" )->item(0);
	$oTEI->setAttributeNS('http://www.w3.org/XML/1998/namespace', 'xml:lang', $szLanguage );
      }

    public function setOddLanguage( $szOddLanguage )
      {
	$this->getXPath( $oXPath );
	$oTEI = $oXPath->query( "//tei:schemaSpec" )->item(0);
	$oTEI->setAttribute('targetLang', $szOddLanguage );
      }

    public function setDocLanguage( $szDocLanguage )
      {
	$this->getXPath( $oXPath );
	$oTEI = $oXPath->query( "//tei:schemaSpec" )->item(0);
	$oTEI->setAttribute('docLang', $szDocLanguage );
	$_SESSION['docLang'] = $szDocLanguage;
      }


    // #####################################################################
    // --- Little Helpers
    // #####################################################################

    protected function getDocDom( &$oDOC )
      {
	$oXSL = new domDocument();
	$oXSL->load( roma_StylesheetDir . '/odds/odd2odd.xsl' );
	$oProc = new XsltProcessor();
	$oProc->importStylesheet( $oXSL );
	$oProc->setParameter( null, 'TEISERVER', roma_xquery_server);
//DEBUG	$oProc->setParameter( null, 'localsource', roma_local_p5);
	$oProc->setParameter( null, 'TEIC', 'true');
	$this->getOddLanguage( $szOddLanguage );
        if ($szOddLanguage=='en') 
     	  { 
	  $oDOC = $oProc->transformToDoc( $this ); }
	else
	   {
              $oXSL2 = new domDocument();
              $oXSL2->load( roma_StylesheetDir . '/odds/translate-odd.xsl' );
              $oProc2 = new XsltProcessor();
 	      $oProc2->setParameter( null, 'TEIC', 'true');	
//DEBUG	      $oProc2->setParameter( null, 'localsource', roma_local_p5);
              $oProc2->setParameter( null, 'TEISERVER', roma_xquery_server);
              $oProc2->setParameter( null, 'lang', $szOddLanguage );
              $oProc2->importStylesheet( $oXSL2 );
              $oDOC = $oProc2->transformToDoc($oProc->transformToDoc($this )); 
	 }
      }

    protected function getSchemaRNGDom( &$oRNG )
      {
	$this->getDocDom( $oDOC );
        $oXSL = new domDocument();
 	$oXSL->load( roma_StylesheetDir . '/odds/odd2relax.xsl'  );
	$oProc = new XsltProcessor();
	$oProc->importStylesheet( $oXSL );
//DEBUG        $oProc->setParameter( null, 'localsource', roma_local_p5);
	$oProc->setParameter( null, 'TEIC', 'true');
        $oProc->setParameter( null, 'doclang', $_SESSION['docLang'] );
	$oProc->setParameter( null, 'displayMode', 'rnc' );
	$oProc->setParameter( null, 'outputDir', '-' );
	$oRNG = $oProc->transformToDoc( $oDOC );
//DEBUG	$szSpecialFile = '/tmp/ROMA.xml';    
//DEBUG	file_put_contents( $szSpecialFile , $oDOC->saveXML());

      } 

    public function loadProgressBar()
      {
	echo '<script>';
	echo "showPgb();";
	echo '</script>';
	flush();
      }

    public function updateProgressBar( $nPercentage, $bIncrease = false, $nMax = 0 )
      {
	static $nLast = 0;
	if ( $bIncrease )
	  {
	    if ( $nMax > ( $nPercentage + $nLast ) )
	      {
		$nPercentage += $nLast;
	      }
	    else
	      {
		$nPercentage = $nLast;
	      }
	  }
	
	echo '<script>';
	echo "setPgb('pgbMain', '{$nPercentage}');";
	echo '</script>';
	flush();

	$nLast = $nPercentage;
      }

    // #####################################################################
    // --- Creating some documentation versions
    // #####################################################################
    public function getTeiLiteDom( &$oTeiLite )
      {
	if ( $this->bBar )
        $this->m_oRomaDom->updateProgressBar( '55' );
	$this->getDocDom( $oDOC );
        $oXSL = new domDocument();
 	$oXSL->load( roma_StylesheetDir . '/odds/odd2lite.xsl'  );
	$oProc = new XsltProcessor();
	$oProc->importStylesheet( $oXSL );
	$oProc->setParameter( null, 'TEIC', 'true');
	$oProc->setParameter( null, 'displayMode', 'rnc' );
	
	$oTeiLite = $oProc->transformToDoc( $oDOC );
      }

    public function outputPlain( &$szDoc, $bBar = false )
      {
	if ( $bBar )
	  {
	    $this->loadProgressBar();
	    $this->updateProgressBar( '30' );
	  }
	$this->getDocDom( $oDOC );
	if ( $bBar )
	    $this->updateProgressBar( '80' );
//	$oTidy = new tidy();
//	$aszOptions = array( 'indent' => true,
//			     'indent-spaces' => 1,
//			     'wrap' => 72,
//			     'input-xml' => true,
//			     'output-xml' => true
//			     );
//	$oTidy->parseString( $oDOC->SaveXML(), $aszOptions );

//	$oTidy->cleanRepair();
	$szDoc = $oDOC->SaveXML();

	if ( $bBar )
	    $this->updateProgressBar( '100' );
      }


    public function outputTeiLite( &$szTeiLite )
      {
	$this->getTeiLiteDom( $oTeiLiteDom );
	if ( $this->bBar )
	    $this->m_oRomaDom->updateProgressBar( '70' );
	
//	$oTidy = new tidy();
//	$aszOptions = array( 'indent' => false,
//			     'input-xml' => true,
//			     'output-xml' => true
//			     );
//	$oTidy->parseString( $oTeiLiteDom->SaveXML(), $aszOptions );
	if ( $this->bBar )
	    $this->m_oRomaDom->updateProgressBar( '80' );

//	$oTidy->cleanRepair();
	$szTeiLite = $oTeiLiteDom->SaveXML();

	if ( $this->bBar )
	    $this->m_oRomaDom->updateProgressBar( '100' );
      }

    public function outputLatex( &$szLatex )
      {
	$this->getTeiLiteDom( $oTeiLiteDom );
	if ( $this->bBar )
	  $this->m_oRomaDom->updateProgressBar( '70' );

	$oXSL = new domDocument();
	$oXSL->load( roma_StylesheetDir . '/latex/tei.xsl'  );
	$oProc = new XsltProcessor();
	$oProc->importStylesheet( $oXSL );
	
	if ( $this->bBar )
	  $this->m_oRomaDom->updateProgressBar( '85' );
	
	$szLatex = $oProc->transformToXML( $oTeiLiteDom );
	
	if ( $this->bBar )
	  $this->m_oRomaDom->updateProgressBar( '100' );
      }

    public function outputPdfLatex( &$szPdf )
      {
	$this->getTeiLiteDom( $oTeiLiteDom );
	if ( $this->bBar )
	  $this->m_oRomaDom->updateProgressBar( '70' );

	
	$oXSL = new domDocument();
	$oXSL->load( roma_StylesheetDir . '/latex/tei.xsl'  );
	$oProc = new XsltProcessor();
	$oProc->importStylesheet( $oXSL );

	if ( $this->bBar )
	  $this->m_oRomaDom->updateProgressBar( '75' );

	$szTmp = $oProc->transformToXML( $oTeiLiteDom );
	
	if ( $this->bBar )
	  $this->m_oRomaDom->updateProgressBar( '80' );

	//Save File
	$szID = md5( uniqid(rand(), true ) );
	
	$szInputFile = roma_temporaryFilesDir . '/' . $szID . '.tex';    
	$szOutputFile = roma_temporaryFilesDir . '/' . $szID . '.pdf';    
	
	file_put_contents( $szInputFile , $szTmp );

	if ( $this->bBar )
	  $this->m_oRomaDom->updateProgressBar( '85' );
	
	$szCurrentDir = getcwd();
	chdir( roma_temporaryFilesDir );
	exec( roma_pdflatex . ' -interaction=nonstopmode ' . $szInputFile );
	if ( $this->bBar )
	  $this->m_oRomaDom->updateProgressBar( '90' );

	exec( roma_pdflatex . ' -interaction=nonstopmode ' . $szInputFile );
	chdir( $szCurrentDir );

	if ( $this->bBar )
	  $this->m_oRomaDom->updateProgressBar( '95' );
	
	$szPdf = join( '', file( $szOutputFile ) );
	
	unlink( $szInputFile );
	unlink( $szOutputFile );

	if ( $this->bBar )
	  $this->m_oRomaDom->updateProgressBar( '100' );
      }

    public function outputPDF( &$szPdf )
      {
	$this->getTeiLiteDom( $oTeiLiteDom );

	if ( $this->bBar )
	  $this->m_oRomaDom->updateProgressBar( '60' );

	$oXSL = new domDocument();
	$oXSL->load( roma_StylesheetDir . '/fo/tei.xsl' );
	
	$oProc = new XsltProcessor();
	$oProc->importStylesheet( $oXSL );
	$oTmpDom = $oProc->transformToDoc( $oTeiLiteDom );

	if ( $this->bBar )
	  $this->m_oRomaDom->updateProgressBar( '70' );
	
	//Save File
	$szID = md5( uniqid(rand(), true ) );
	
	$szInputFile = roma_temporaryFilesDir . '/' . $szID . '.fo';    
	$szOutputFile = roma_temporaryFilesDir . '/' . $szID . '.pdf';    
	
	file_put_contents( $szInputFile , $oTmpDom->SaveXML() );

	if ( $this->bBar )
	  $this->m_oRomaDom->updateProgressBar( '80' );

	exec( roma_fop . ' ' . $szInputFile . ' ' . $szOutputFile );

	if ( $this->bBar )
	  $this->m_oRomaDom->updateProgressBar( '90' );

	$szPdf = join( '', file( $szOutputFile ) );
	
	unlink( $szInputFile );
	unlink( $szOutputFile );

	if ( $this->bBar )
	  $this->m_oRomaDom->updateProgressBar( '100' );
      }

    public function outputDVI ( &$szDVI )
      {
	$this->getTeiLiteDom( $oTeiLiteDom );

	if ( $this->bBar )
	  $this->m_oRomaDom->updateProgressBar( '60' );

	$oXSL = new domDocument();
	$oXSL->load( roma_StylesheetDir . '/latex/tei.xsl');
	
	$oProc = new XsltProcessor();
	$oProc->importStylesheet( $oXSL );
	$szTmp = $oProc->transformToXML( $oTeiLiteDom );

	if ( $this->bBar )
	  $this->m_oRomaDom->updateProgressBar( '70' );
	
	//Save File
	$szID = md5( uniqid(rand(), true ) );
	
	$szInputFile = roma_temporaryFilesDir . '/' . $szID . '.tex';    
	$szOutputFile = roma_temporaryFilesDir . '/' . $szID . '.dvi';    
	
	file_put_contents( $szInputFile , $szTmp );

	if ( $this->bBar )
	  $this->m_oRomaDom->updateProgressBar( '80' );

	$szCurrentDir = getcwd();
	chdir( roma_temporaryFilesDir );
	exec( roma_latex . ' -interaction=nonstopmode ' . $szInputFile );
	if ( $this->bBar )
	  $this->m_oRomaDom->updateProgressBar( '90' );

	exec( roma_latex . ' -interaction=nonstopmode ' . $szInputFile );
	chdir( $szCurrentDir );

	if ( $this->bBar )
	  $this->m_oRomaDom->updateProgressBar( '95' );
	
	$szDVI = join( '', file( $szOutputFile ) );
	
	unlink( $szInputFile );
	unlink( $szOutputFile );

	if ( $this->bBar )
	  $this->m_oRomaDom->updateProgressBar( '100' );
      }

    public function outputHTML ( &$szHTML )
      {
	$this->getDocDom( $oDOC );
	if ( $this->bBar )
	  $this->m_oRomaDom->updateProgressBar( '55' );
        $oXSL = new domDocument();
 	$oXSL->load( roma_StylesheetDir . '/html/odd2html.xsl'  );

	$oProc = new XsltProcessor();
	$oProc->importStylesheet( $oXSL );
	$oProc->setParameter( null, 'TEIC', 'true');	
	$oProc->setParameter( null, 'displayMode', 'rnc' );
	$oProc->setParameter( null, 'STDOUT', 'true' );
	$oProc->setParameter( null, 'splitLevel', '-1' );

	if ( $this->bBar )
	  $this->m_oRomaDom->updateProgressBar( '80' );

	$szHTML = $oProc->transformToDoc( $oDOC )->SaveHTML();

	if ( $this->bBar )
	  $this->m_oRomaDom->updateProgressBar( '100' );
      }

    // #####################################################################
    // --- Creating some Schema
    // #####################################################################

    public function createSchemaRNG( &$szRNG, $bBar = false )
      {
	if ( $bBar )
	  {
	    $this->loadProgressBar();
	    $this->updateProgressBar( '30' );
	  }
	$this->getSchemaRNGDom( $oRNG );
	if ( $bBar )
	    $this->updateProgressBar( '80' );
//	$oTidy = new tidy();
//	$aszOptions = array( 'indent' => true,
//			     'indent-spaces' => 1,
//			     'wrap' => 72,
//			     'input-xml' => true,
//			     'output-xml' => true
//			     );
//	$oTidy->parseString( $oRNG->SaveXML(), $aszOptions );

//	$oTidy->cleanRepair();
//	$szRNG = $oTidy->value;
	$szRNG = $oRNG->SaveXML();
	if ( $bBar )
	    $this->updateProgressBar( '100' );
      }

    public function createSchemaRNC( &$szRNC, $bBar = false )
      {
	if ( $bBar )
	  {
	    $this->loadProgressBar();
	    $this->updateProgressBar( '30' );
	  }
	$this->getSchemaRNGDom( $oRNG );
	if ( $bBar )
	    $this->updateProgressBar( '50' );

//	$oTidy = new tidy();
//	$aszOptions = array( 'indent' => true,
//			     'indent-spaces' => 1,
//			     'wrap' => 72,
//			     'input-xml' => true,
//			     'output-xml' => true
//			     );
//	$oTidy->parseString( $oRNG->SaveXML(), $aszOptions );

//	$oTidy->cleanRepair();

	//Save File
	$szID = md5( uniqid(rand(), true ) );
	
	$szInputFile = roma_temporaryFilesDir . '/' . $szID . '.tmp';    
	$szOutputFile = roma_temporaryFilesDir . '/' . $szID . '.rnc';    
	file_put_contents( $szInputFile , $oRNG->SaveXML());

	if ( $bBar )
	    $this->updateProgressBar( '70' );

	ob_start();
	System( roma_trang . ' -I rng -O rnc ' . $szInputFile . ' ' . $szOutputFile  . ' 2>&1');
	$szError = ob_get_clean();
	ob_end_clean();

	if ( $bBar )
	    $this->updateProgressBar( '90' );


	if ( file_exists( $szOutputFile ) )
	  {
	    $szRNC = join( '', file( $szOutputFile ) );
	    unlink( $szOutputFile );
	  }
	unlink( $szInputFile );

	if ( $bBar )
	    $this->updateProgressBar( '100' );

	return $szError;
      }

    public function createSchemaXSD( &$szXSD, $bBar = false )
      {
	if ( $bBar )
	  {
	    $this->loadProgressBar();
	    $this->updateProgressBar( '30' );
	  }
	$this->getSchemaRNGDom( $oRNG );
	if ( $bBar )
	    $this->updateProgressBar( '50' );

	//Save File
	$szID = md5( uniqid(rand(), true ) );
	
	$szInputFile = roma_temporaryFilesDir . '/' . $szID . '.tmp';    
	$szOutputFile = roma_temporaryFilesDir . '/' . $szID . '.xsd';    

	file_put_contents( $szInputFile ,$oRNG ->SaveXML());
	

	if ( $bBar )
	    $this->updateProgressBar( '70' );
	
	ob_start();
	System( roma_trang . 
	' -I rng -O xsd -o disable-abstract-elements ' . 
	$szInputFile . ' ' . $szOutputFile  . ' 2>&1;' .
	'perl -p -i -e	"s+\"xml.xsd\"+\"http://www.w3.org/2004/10/xml.xsd\"+" ' .
        $szOutputFile);
	$szError = ob_get_clean();
	ob_end_clean();

	if ( $bBar )
	    $this->updateProgressBar( '90' );

	if ( file_exists( $szOutputFile ) )
	  { 
	    $szXSD = join( '', file( $szOutputFile ) );
	    unlink( $szOutputFile );
	  }

	unlink( $szInputFile );

	if ( $bBar )
	    $this->updateProgressBar( '100' );

	return $szError;
      }

    public function createSchemaDTD( &$szDTD, $bBar = false )
      {
	if ( $bBar )
	  {
	    $this->loadProgressBar();
	    $this->updateProgressBar( '30' );
	  }
	$this->getDocDom( $oDOC );
        $oXSL = new domDocument();
 	$oXSL->load( roma_StylesheetDir . '/odds/odd2dtd.xsl'  );
	$oProc = new XsltProcessor();
	$oProc->importStylesheet( $oXSL );
	$oProc->setParameter( null, 'TEIC', 'true');	
	$oProc->setParameter( null, 'outputDir', '-' );
	if ( $bBar )
	    $this->updateProgressBar( '70' );
	$szDTD = $oProc->transformToXML( $oDOC );
	if ( $bBar )
	    $this->updateProgressBar( '100' );
	return $szError;
      }

  }

</script>
